/*

    this source file is a part of 
    lp_parse - a preprocessor for MILP models
    Copyright (C) 1996  Ari Jolma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    for more information please contact 
    Ari Jolma
    Helsinki University of Technology
    Laboratory of Water Resources Engineering
    Building W
    02150 Espoo, Finland
    E-mail: Ari.Jolma@hut.fi

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "hash.h"

/*--- hashpjw.c -------------------------------------------------
 *  An adaptation of Peter Weinberger's (PJW) generic hashing
 *  algorithm based on Allen Holub's version.
 *
 *  Accepts a pointer to a datum to be hashed and returns an
 *  unsigned integer. 
 *-------------------------------------------------------------*/

#include <limits.h>

#define BITS_IN_int             ( sizeof(int) * CHAR_BIT )
#define THREE_QUARTERS	        ((int) ((BITS_IN_int * 3) / 4))
#define ONE_EIGHTH		((int) (BITS_IN_int / 8))
#define HIGH_BITS		( ~((unsigned int)(~0) >> ONE_EIGHTH ))

unsigned int HashPJW ( const char * datum )
{
  unsigned int hash_value, i;

  for ( hash_value = 0; *datum; ++datum )
    {
      hash_value = ( hash_value << ONE_EIGHTH ) + *datum;
      if (( i = hash_value & HIGH_BITS ) != 0 )
	hash_value =
	  ( hash_value ^ ( i >> THREE_QUARTERS )) &
	~HIGH_BITS;
    }

  return ( hash_value );
}


/*--- elfhash.c ------------------------------------------------
 *  The published hash algorithm used in the UNIX ELF format
 *  for object files.
 *
 *  Accepts a pointer to a string to be hashed and returns an
 *  unsigned long. Algorithm is similar to that implemented
 *  in HashPJW.                           
 *-------------------------------------------------------------*/

unsigned long ElfHash ( const unsigned char *name )
{
  
  unsigned long   h = 0, g;

  while ( *name )
    {
      h = ( h << 4 ) + *name++;
      if (( g = h & 0xF0000000 ))
	h ^= g >> 24;

      h &= ~g;
    }
  return h; 
    
}

int ATJHash(const char *name )
{
  
  int h = 0;
  while (*name) {
    h += (int)*name;
    name++;
  }
  return h;
}  


void insert_hash_entry(HASH_ENTRY *hash_entry,
		       HASH_ENTRY *hash_table[], 
		       int hash_key) 
{

  HASH_ENTRY **h = &hash_table[hash_key];

#ifdef DEBUG
  /*printf("ihe %s %i\n", hash_entry->name, hash_key);*/
#endif

  while (*h)
    h = (HASH_ENTRY **)&((*h)->next);
  *h = hash_entry;

}

HASH_ENTRY *search_hash_entry(char *name,
			      HASH_ENTRY *hash_table[], 
			      int hash_key) 
{
  HASH_ENTRY *h = hash_table[hash_key];

#ifdef DEBUG
  /*printf("she %s %i ", name, hash_key);*/
#endif

  while (h) {

#ifdef DEBUG
    /*printf("she srch: %s", h->name);*/
#endif

    if (strcmp(h->name, name) == 0) {

#ifdef DEBUG
      /*printf("found\n");*/
#endif

      return h;
    }
    h = h->next;
  }

#ifdef DEBUG
  /*printf("not found\n");*/
#endif

  return NULL;
}

